import jQuery from 'jquery';
import _ from 'lodash';
import { getAttrs } from './xmlParser';
import PropsWatcher from './PropsWatcher';

const createdID = () => {
  let id = 0;
  return () => {
    return id++;
  }
};
let JqComponent_ID = createdID();

function create$$($dom) {
  return function find(selector) {
    let childen = $dom.find(selector);
    $dom.parent().find(selector).each((index, $d) => {
      if ($dom.get(0) === $d) {
        childen.push($d)
      }
    });
    return childen;
  };
}

function setAttr($fragment, attrs) {
  Object.keys(attrs).forEach(key => {
    let values = attrs[key];
    values.forEach(v => {
      let lastedVal = $fragment.attr(key);
      let attrVals = lastedVal ? `${lastedVal} ${v}` : v;
      $fragment.attr(key, attrVals);
    });
  });
}

function mountedStyles($styles, tagName, jqcId) {
  jQuery("head").append($styles)
  setComponentInfo2Dom($styles, _.toLower(tagName), jqcId);
}

function proxy$$($$) {
  var handler = {
    get(target, key) {
      if (!target[key]) {
        return jQuery[key];
      }
      return target[key];
    }
  };
  return new Proxy($$, handler);
}



const setComponentInfo2Dom = (dom, tagName, id) => {
  // jQuery(dom).attr('component-name', tagName)
  jQuery(dom).attr('component-id', id)
}

const eventHooks = function (id, dom, attrs) {
  Object.keys(attrs).forEach(attr => {
    if (_.startsWith(attr, 'on')) {
      dom[attr] = () => {
        let lisfn = attrs[attr][0];
        var fnName = lisfn.replace(/\(.+?\)/g, '')
        var arg = lisfn.match(/\((.+?)\)/g);
        arg = arg[0].replace(/\(|\)/g, '').split(',').map(item => _.trim(item));
        listenProps.emit(id, fnName, arg, dom);
      }
    }
  })
}

export let allComponents = {};

let globalComponents = {};

export const listenProps = new PropsWatcher();

export const setGlobalComponent = function (name, jqc) {
  globalComponents[name] = jqc;
  return globalComponents;
};

export const removeGlobalComponent = function (name) {
  delete globalComponents[name];
  return globalComponents;
}
export default class JqComponent {
  constructor(options = {}) {
    this.options = options;
    let { template, styles, components, ready } = this.options;
    this.template = template;
    this.$fragment = jQuery(template);
    this.components = globalComponents;
    this.$styles = jQuery(styles);
    this.ready = ready;
    this.id = JqComponent_ID();

    Object.keys(components || {}).forEach(key => {
      this.components[_.toUpper(key)] = components[key];
    });

    allComponents[this.id] = this;
  }

  render(root) {
    const tagName = root.tagName;
    if (tagName) {
      const options = this.components[tagName];
      const attrs = getAttrs(root);
   
      if (options) {
        const jqc = new JqComponent(options);
        setAttr(jqc.$fragment, attrs);

        root.parentNode.replaceChild(jqc.$fragment.get(0), root)
        setComponentInfo2Dom(jqc.$fragment, tagName, jqc.id);
        mountedStyles(jqc.$styles, tagName, jqc.id);
        jqc.render(jqc.$fragment.get(0));

        if (_.isFunction(jqc.ready)) {
          const $$ = create$$(jqc.$fragment);
          const watcher = (key, val) => listenProps.on(jqc.id, key, val);
          $$.on = watcher;
          $$.el = jqc.$fragment;
          jqc.ready.apply(jqc.$fragment, [proxy$$($$)]);
        }
      }
    }
    root.childNodes.forEach(child => {
      this.render(child);
    });
  }
};